import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestWindowWithCount(unittest.TestCase):
    def test_window_with_count_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(100, 1), on_next(210, 2), on_next(240, 3), on_next(280, 4), on_next(320, 5), on_next(350, 6), on_next(380, 7), on_next(420, 8), on_next(470, 9), on_completed(600))
        def create():
            def proj(w, i):
                return w.pipe(ops.map(lambda x: str(i) + ' ' + str(x)))
            return xs.pipe(ops.window_with_count(3, 2), ops.map_indexed(proj), ops.merge_all())
        results = scheduler.start(create)

        assert results.messages == [on_next(210, "0 2"), on_next(240, "0 3"), on_next(280, "0 4"), on_next(280, "1 4"), on_next(320, "1 5"), on_next(350, "1 6"), on_next(350, "2 6"), on_next(380, "2 7"), on_next(420, "2 8"), on_next(420, "3 8"), on_next(470, "3 9"), on_completed(600)]
        assert xs.subscriptions == [subscribe(200, 600)]

    def test_window_with_count_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(100, 1), on_next(210, 2), on_next(240, 3), on_next(280, 4), on_next(320, 5), on_next(350, 6), on_next(380, 7), on_next(420, 8), on_next(470, 9), on_completed(600))

        def create():
            def proj(w, i):
                return w.pipe(ops.map(lambda x: str(i) + ' ' + str(x)))
            return xs.pipe(ops.window_with_count(3, 2), ops.map_indexed(proj), ops.merge_all())

        results = scheduler.start(create, disposed=370)
        assert results.messages == [on_next(210, "0 2"), on_next(240, "0 3"), on_next(280, "0 4"), on_next(280, "1 4"), on_next(320, "1 5"), on_next(350, "1 6"), on_next(350, "2 6")]
        assert xs.subscriptions == [subscribe(200, 370)]

    def test_window_with_count_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(100, 1), on_next(210, 2), on_next(240, 3), on_next(280, 4), on_next(320, 5), on_next(350, 6), on_next(380, 7), on_next(420, 8), on_next(470, 9), on_error(600, ex))

        def create():
            def mapper(w, i):
                def mapping(x):
                    return "%s %s" % (i, x)
                return w.pipe(ops.map(mapping))
            return xs.pipe(ops.window_with_count(3, 2), ops.map_indexed(mapper), ops.merge_all())

        results = scheduler.start(create)

        assert results.messages == [on_next(210, "0 2"), on_next(240, "0 3"), on_next(280, "0 4"), on_next(280, "1 4"), on_next(320, "1 5"), on_next(350, "1 6"), on_next(350, "2 6"), on_next(380, "2 7"), on_next(420, "2 8"), on_next(420, "3 8"), on_next(470, "3 9"), on_error(600, ex)]
        assert xs.subscriptions == [subscribe(200, 600)]



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestFind(unittest.TestCase):

    def test_find_never(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1)
        )

        def create():
            return xs.pipe(ops.find(lambda x, i, s: True))

        res = scheduler.start(create)

        assert res.messages == []

    def test_find_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_completed(210)
        )

        def create():
            return xs.pipe(ops.find(lambda x, i, s: True))

        res = scheduler.start(create)

        assert res.messages == [
            on_next(210, None),
            on_completed(210)]

    def test_find_single(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_completed(220)
        )

        def create():
            return xs.pipe(ops.find(lambda x, i, s: x == 2))
        res = scheduler.start(create)

        assert res.messages == [
            on_next(210, 2),
            on_completed(210)]

    def test_find_notfound(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_completed(220)
        )

        def create():
            return xs.pipe(ops.find(lambda x, i, s: x == 3))
        res = scheduler.start(create)

        assert res.messages == [
            on_next(220, None),
            on_completed(220)]

    def test_find_Error(self):
        ex = Exception('error')
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_error(220, ex)
        )

        def create():
            return xs.pipe(ops.find(lambda x, i, s: x == 3))
        res = scheduler.start(create)

        assert res.messages == [
            on_error(220, ex)]

    def test_find_throws(self):
        ex = 'error'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_completed(220)
        )

        def create():
            def predicate(x, i, source):
                raise Exception(ex)
            return xs.pipe(ops.find(predicate))
        res = scheduler.start(create)

        assert res.messages == [
            on_error(210, ex)]



import unittest
import pytest

import rx
from rx.testing import TestScheduler, ReactiveTest


on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestGenerateWithRelativeTime(unittest.TestCase):
    def test_generate_timespan_finite(self):
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: x <= 3,
                lambda x: x + 1,
                lambda x: x + 1
            )

        results = scheduler.start(create)
        assert results.messages == [on_next(201, 0), on_next(
            203, 1), on_next(206, 2), on_next(210, 3), on_completed(210)]

    def test_generate_timespan_throw_condition(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: _raise(ex),
                lambda x: x + 1,
                lambda x: x + 1
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_generate_timespan_throw_iterate(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: _raise(ex),
                lambda x: x + 1
            )

        results = scheduler.start(create)
        assert results.messages == [on_next(201, 0), on_error(201, ex)]

    def test_generate_timespan_throw_timemapper(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: x + 1,
                lambda x: _raise(ex)
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_generate_timespan_dispose(self):
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: x + 1,
                lambda x: x + 1
            )

        results = scheduler.start(create, disposed=210)
        assert results.messages == [on_next(201, 0), on_next(203, 1), on_next(206, 2)]

    def test_generate_datetime_offset_finite(self):
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: x <= 3,
                lambda x: x + 1,
                lambda x: x + 1
            )

        results = scheduler.start(create)
        assert results.messages == [
            on_next(201, 0), on_next(203, 1), on_next(206, 2), on_next(210, 3), on_completed(210)
        ]

    def test_generate_datetime_offset_throw_condition(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: _raise(ex),
                lambda x: x + 1,
                lambda x: x + 1)

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_generate_datetime_offset_throw_iterate(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: _raise(ex),
                lambda x: x + 1)

        results = scheduler.start(create)
        assert results.messages == [on_next(201, 0), on_error(201, ex)]

    def test_generate_datetime_offset_throw_time_mapper(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: x + 1,
                lambda x: _raise(ex)
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_generate_datetime_offset_dispose(self):
        scheduler = TestScheduler()

        def create():
            return rx.generate_with_relative_time(
                0,
                lambda x: True,
                lambda x: x + 1,
                lambda x: x + 1
            )

        results = scheduler.start(create, disposed=210)
        assert results.messages == [on_next(201, 0), on_next(203, 1), on_next(206, 2)]



import unittest

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestToDict(unittest.TestCase):

    def test_to_dict_completed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(220, 2),
            on_next(330, 3),
            on_next(440, 4),
            on_next(550, 5),
            on_completed(660)
        )

        def create():
            return xs.pipe(ops.to_dict(lambda x: x * 2, lambda x: x * 4))

        res = scheduler.start(create)
        assert res.messages == [
            on_next(660, {4: 8, 6: 12, 8: 16, 10: 20}),
            on_completed(660)]

        assert xs.subscriptions == [
            subscribe(200, 660)]

    def test_to_dict_error(self):
        scheduler = TestScheduler()

        ex = Exception()

        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(220, 2),
            on_next(330, 3),
            on_next(440, 4),
            on_next(550, 5),
            on_error(660, ex)
        )

        def create():
            return xs.pipe(ops.to_dict(lambda x: x * 2, lambda x: x * 4))

        res = scheduler.start(create)

        assert res.messages == [
            on_error(660, ex)]

        assert xs.subscriptions == [
           subscribe(200, 660)]


    def test_to_dict_keymapperthrows(self):
        scheduler = TestScheduler()

        ex = Exception()

        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(220, 2),
            on_next(330, 3),
            on_next(440, 4),
            on_next(550, 5),
            on_completed(600)
          )

        def create():
            def key_mapper(x):
                if x < 4:
                    return x * 2
                else:
                    raise ex
            return xs.pipe(ops.to_dict(key_mapper, lambda x: x * 4))

        res = scheduler.start(create)

        assert res.messages == [
            on_error(440, ex)]

        assert xs.subscriptions == [
            subscribe(200, 440)]

    def test_to_dict_elementmapperthrows(self):
        scheduler = TestScheduler()

        ex = Exception()

        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(220, 2),
            on_next(330, 3),
            on_next(440, 4),
            on_next(550, 5),
            on_completed(600)
        )

        def value_mapper(x):
            if x < 4:
                return x * 4
            else:
                raise ex

        def create():
            return xs.pipe(ops.to_dict(lambda x: x * 2 , value_mapper))

        res = scheduler.start(create)

        assert res.messages == [
            on_error(440, ex)]

        assert xs.subscriptions == [
            subscribe(200, 440)]

    def test_to_dict_disposed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(220, 2),
            on_next(330, 3),
            on_next(440, 4),
            on_next(550, 5)
        )

        def create():
            return xs.pipe(ops.to_dict(lambda x: x * 2, lambda x: x * 4))

        res = scheduler.start(create)

        assert res.messages == []

        assert xs.subscriptions == [
            subscribe(200, 1000)]



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestToAsync(unittest.TestCase):
    def test_to_async_context(self):
        class Context:
            def __init__(self):
                self.value = 42

            def func(self, x):
                return self.value + x

        scheduler = TestScheduler()

        def create():
            context = Context()
            return rx.to_async(context.func, scheduler)(42)

        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 84),
            on_completed(200)]

    def test_to_async0(self):
        scheduler = TestScheduler()

        def create():
            def func():
                return 0

            return rx.to_async(func, scheduler)()

        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 0),
            on_completed(200)]

    def test_to_async1(self):
        scheduler = TestScheduler()

        def create():
            def func(x):
                return x

            return rx.to_async(func, scheduler)(1)

        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 1),
            on_completed(200)]

    def test_to_async2(self):
        scheduler = TestScheduler()

        def create():
            def func(x, y):
                return x + y

            return rx.to_async(func, scheduler)(1, 2)

        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 3),
            on_completed(200)]

    def test_to_async3(self):
        scheduler = TestScheduler()

        def create():
            def func(x, y, z):
                return x + y + z

            return rx.to_async(func, scheduler)(1, 2, 3)

        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 6),
            on_completed(200)]

    def test_to_async4(self):
        scheduler = TestScheduler()

        def create():
            def func(a, b, c, d):
                return a + b + c + d

            return rx.to_async(func, scheduler)(1, 2, 3, 4)
        res = scheduler.start(create)

        assert res.messages == [
            on_next(200, 10),
            on_completed(200)]

    def test_to_async_error0(self):
        ex = Exception()

        scheduler = TestScheduler()

        def create():
            def func():
                raise ex
            return rx.to_async(func, scheduler)()

        res = scheduler.start(create)

        assert res.messages == [
            on_error(200, ex)]

    def test_to_async_error1(self):
        ex = Exception()

        scheduler = TestScheduler()

        def create():
            def func(a):
                raise ex
            return rx.to_async(func, scheduler)(1)

        res = scheduler.start(create)

        assert res.messages == [
            on_error(200, ex)]

    def test_to_async_error2(self):
        ex = Exception()

        scheduler = TestScheduler()

        def create():
            def func(a, b):
                raise ex
            return rx.to_async(func, scheduler)(1, 2)

        res = scheduler.start(create)

        assert res.messages == [
            on_error(200, ex)]

    def test_to_async_error3(self):
        ex = Exception()

        scheduler = TestScheduler()

        def create():
            def func(a, b, c):
                raise ex
            return rx.to_async(func, scheduler)(1, 2, 3)

        res = scheduler.start(create)

        assert res.messages == [
            on_error(200, ex)]

    def test_to_async_error4(self):
        ex = Exception()

        scheduler = TestScheduler()

        def create():
            def func(a, b, c, d):
                raise ex
            return rx.to_async(func, scheduler)(1, 2, 3, 4)

        res = scheduler.start(create)

        assert res.messages == [
            on_error(200, ex)]



import unittest

from rx import operators as _
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestLast(unittest.TestCase):

    def test_last_async_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(_.last())

        res = scheduler.start(create=create)

        def predicate(e):
            return e is not None

        assert [on_error(250, predicate)] == res.messages
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_async_one(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(150, 1), on_next(210, 2), on_completed(250))

        def create():
            return xs.pipe(_.last())

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 2), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_async_many(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(150, 1), on_next(210, 2), on_next(220, 3),
                on_completed(250))

        def create():
            return xs.pipe(_.last())

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 3), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_async_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))

        def create():
            return xs.pipe(_.last())

        res = scheduler.start(create=create)

        assert res.messages == [on_error(210, ex)]
        assert xs.subscriptions == [subscribe(200, 210)]

    def test_last_async_predicate(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(150, 1), on_next(210, 2), on_next(220, 3),
                on_next(230, 4), on_next(240, 5), on_completed(250))

        def create():
            def predicate(x):
                return x % 2 == 1
            return xs.pipe(_.last(predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 5), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_async_predicate_none(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(150, 1), on_next(210, 2), on_next(220, 3),
                on_next(230, 4), on_next(240, 5), on_completed(250))

        def create():
            def predicate(x):
                return x > 10
            return xs.pipe(_.last(predicate))

        res = scheduler.start(create=create)

        def predicate(e):
            return e is not None

        assert [on_error(250, predicate)] == res.messages
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_async_predicate_on_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))

        def create():
            def predicate(x):
                return x % 2 == 1

            return xs.pipe(_.last(predicate))
        res = scheduler.start(create=create)

        assert res.messages == [on_error(210, ex)]
        assert xs.subscriptions == [subscribe(200, 210)]

    def test_last_async_predicate_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(150, 1), on_next(210, 2), on_next(220, 3),
                on_next(230, 4), on_next(240, 5), on_completed(250))

        def create():
            def predicate(x):
                if x < 4:
                    return x % 2 == 1
                else:
                    raise Exception(ex)

            return xs.pipe(_.last(predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_error(230, ex)]
        assert xs.subscriptions == [subscribe(200, 230)]



import unittest
import pytest

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestMerge(unittest.TestCase):
    def test_merge_never2(self):
        scheduler = TestScheduler()
        n1 = rx.never()
        n2 = rx.never()

        def create():
            return rx.merge(n1, n2)

        results = scheduler.start(create)
        assert results.messages == []

    def test_merge_never3(self):
        scheduler = TestScheduler()
        n1 = rx.never()
        n2 = rx.never()
        n3 = rx.never()

        def create():
            return rx.merge(n1, n2, n3)

        results = scheduler.start(create)
        assert results.messages == []

    def test_merge_empty2(self):
        scheduler = TestScheduler()
        e1 = rx.empty()
        e2 = rx.empty()

        def create():
            return rx.merge(e1, e2)

        results = scheduler.start(create)
        assert results.messages == [on_completed(200)]

    def test_merge_empty3(self):
        scheduler = TestScheduler()
        e1 = rx.empty()
        e2 = rx.empty()
        e3 = rx.empty()

        def create():
            return rx.merge(e1, e2, e3)

        results = scheduler.start(create)
        assert results.messages == [on_completed(200)]

    def test_merge_empty_delayed2_right_last(self):
        scheduler = TestScheduler()
        l_msgs = [on_next(150, 1), on_completed(240)]
        r_msgs = [on_next(150, 1), on_completed(250)]
        e1 = scheduler.create_hot_observable(l_msgs)
        e2 = scheduler.create_hot_observable(r_msgs)

        def create():
            return rx.merge(e1, e2)

        results = scheduler.start(create)
        assert results.messages == [on_completed(250)]

    def test_merge_empty_delayed2_left_last(self):
        scheduler = TestScheduler()
        l_msgs = [on_next(150, 1), on_completed(250)]
        r_msgs = [on_next(150, 1), on_completed(240)]
        e1 = scheduler.create_hot_observable(l_msgs)
        e2 = scheduler.create_hot_observable(r_msgs)

        def create():
            return rx.merge(e1, e2)
        results = scheduler.start(create)
        assert results.messages == [on_completed(250)]

    def test_merge_empty_delayed3_middle_last(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(245)]
        msgs2 = [on_next(150, 1), on_completed(250)]
        msgs3 = [on_next(150, 1), on_completed(240)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)
        e3 = scheduler.create_hot_observable(msgs3)

        def create():
            return rx.merge(e1, e2, e3)

        results = scheduler.start(create)
        assert results.messages == [on_completed(250)]

    def test_merge_empty_never(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(245)]
        e1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(e1, n1)

        results = scheduler.start(create)
        assert results.messages == []

    def test_merge_never_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(245)]
        e1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(n1, e1)

        results = scheduler.start(create)
        assert results.messages == []

    def test_merge_return_never(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_completed(245)]
        r1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(r1, n1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2)]

    def test_merge_never_return(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_completed(245)]
        r1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(n1, r1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2)]

    def test_merge_error_never(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_error(245, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(e1, n1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_error(245, ex)]

    def test_merge_never_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_error(245, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        n1 = rx.never()

        def create():
            return rx.merge(n1, e1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_error(245, ex)]

    def test_merge_empty_return(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(245)]
        msgs2 = [on_next(150, 1), on_next(210, 2), on_completed(250)]
        e1 = scheduler.create_hot_observable(msgs1)
        r1 = scheduler.create_hot_observable(msgs2)

        def create():
            return rx.merge(e1, r1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_completed(250)]

    def test_merge_return_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(245)]
        msgs2 = [on_next(150, 1), on_next(210, 2), on_completed(250)]
        e1 = scheduler.create_hot_observable(msgs1)
        r1 = scheduler.create_hot_observable(msgs2)

        def create():
            return rx.merge(r1, e1)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_completed(250)]

    def test_merge_lots2(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_next(220, 4), on_next(230, 6), on_next(240, 8), on_completed(245)]
        msgs2 = [on_next(150, 1), on_next(215, 3), on_next(225, 5), on_next(235, 7), on_next(245, 9), on_completed(250)]
        o1 = scheduler.create_hot_observable(msgs1)
        o2 = scheduler.create_hot_observable(msgs2)

        def create():
            return rx.merge(o1, o2)

        results = scheduler.start(create).messages
        assert(len(results) == 9)
        for i, result in enumerate(results[:-1]):
            assert(result.value.kind == 'N')
            assert(result.time == 210 + i * 5)
            assert(result.value.value == i + 2)

        assert(results[8].value.kind == 'C' and results[8].time == 250)

    def test_merge_lots3(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_next(225, 5), on_next(240, 8), on_completed(245)]
        msgs2 = [on_next(150, 1), on_next(215, 3), on_next(230, 6), on_next(245, 9), on_completed(250)]
        msgs3 = [on_next(150, 1), on_next(220, 4), on_next(235, 7), on_completed(240)]
        o1 = scheduler.create_hot_observable(msgs1)
        o2 = scheduler.create_hot_observable(msgs2)
        o3 = scheduler.create_hot_observable(msgs3)

        def create():
            return rx.merge(o1, o2, o3)

        results = scheduler.start(create).messages
        assert(len(results) == 9)
        for i, result in enumerate(results[:-1]):
            assert(results[i].value.kind == 'N' and results[i].time == 210 + i * 5 and results[i].value.value == i + 2)

        assert(results[8].value.kind == 'C' and results[8].time == 250)

    def test_merge_error_left(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_error(245, ex)]
        msgs2 = [on_next(150, 1), on_next(215, 3), on_completed(250)]
        o1 = scheduler.create_hot_observable(msgs1)
        o2 = scheduler.create_hot_observable(msgs2)

        def create():
            return rx.merge(o1, o2)

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_next(215, 3), on_error(245, ex)]

    def test_merge_error_causes_disposal(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_error(210, ex)]
        msgs2 = [on_next(150, 1), on_next(220, 1), on_completed(250)]
        source_not_disposed = [False]
        o1 = scheduler.create_hot_observable(msgs1)

        def action():
            source_not_disposed[0] = True

        o2 = scheduler.create_hot_observable(msgs2).pipe(ops.do_action(on_next=action))

        def create():
            return rx.merge(o1, o2)

        results = scheduler.start(create)

        assert results.messages == [on_error(210, ex)]
        assert(not source_not_disposed[0])

    def test_merge_observable_of_observable_data(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(300, scheduler.create_cold_observable(on_next(10, 101), on_next(20, 102), on_next(110, 103), on_next(120, 104), on_next(210, 105), on_next(220, 106), on_completed(230))), on_next(400, scheduler.create_cold_observable(
            on_next(10, 201), on_next(20, 202), on_next(30, 203), on_next(40, 200), on_completed(50))), on_next(500, scheduler.create_cold_observable(on_next(10, 301), on_next(20, 302), on_next(30, 303), on_next(40, 304), on_next(120, 305), on_completed(150))), on_completed(600))

        def create():
            return xs.pipe(ops.merge_all())
        results = scheduler.start(create)
        assert results.messages == [on_next(310, 101), on_next(320, 102), on_next(410, 103), on_next(410, 201), on_next(420, 104), on_next(420, 202), on_next(430, 203), on_next(
            440, 200), on_next(510, 105), on_next(510, 301), on_next(520, 106), on_next(520, 302), on_next(530, 303), on_next(540, 304), on_next(620, 305), on_completed(650)]

    def test_merge_observable_of_observable_data_non_overlapped(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(300, scheduler.create_cold_observable(on_next(10, 101), on_next(20, 102), on_completed(230))), on_next(400, scheduler.create_cold_observable(on_next(10, 201), on_next(
            20, 202), on_next(30, 203), on_next(40, 200), on_completed(50))), on_next(500, scheduler.create_cold_observable(on_next(10, 301), on_next(20, 302), on_next(30, 303), on_next(40, 304), on_completed(50))), on_completed(600))

        def create():
            return xs.pipe(ops.merge_all())

        results = scheduler.start(create)
        assert results.messages == [on_next(310, 101), on_next(320, 102), on_next(410, 201), on_next(420, 202), on_next(
            430, 203), on_next(440, 200), on_next(510, 301), on_next(520, 302), on_next(530, 303), on_next(540, 304), on_completed(600)]

    def test_merge_observable_of_observable_inner_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(300, scheduler.create_cold_observable(on_next(10, 101), on_next(20, 102), on_completed(230))), on_next(400, scheduler.create_cold_observable(on_next(10, 201), on_next(
            20, 202), on_next(30, 203), on_next(40, 200), on_error(50, ex))), on_next(500, scheduler.create_cold_observable(on_next(10, 301), on_next(20, 302), on_next(30, 303), on_next(40, 304), on_completed(50))), on_completed(600))

        def create():
            return xs.pipe(ops.merge_all())

        results = scheduler.start(create)
        assert results.messages == [on_next(310, 101), on_next(320, 102), on_next(
            410, 201), on_next(420, 202), on_next(430, 203), on_next(440, 200), on_error(450, ex)]

    def test_merge_observable_of_observable_outer_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(300, scheduler.create_cold_observable(on_next(10, 101), on_next(20, 102), on_completed(230))), on_next(
            400, scheduler.create_cold_observable(on_next(10, 201), on_next(20, 202), on_next(30, 203), on_next(40, 200), on_completed(50))), on_error(500, ex))

        def create():
            return xs.pipe(ops.merge_all())
        results = scheduler.start(create)
        assert results.messages == [on_next(310, 101), on_next(320, 102), on_next(
            410, 201), on_next(420, 202), on_next(430, 203), on_next(440, 200), on_error(500, ex)]

    def test_mergeconcat_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, scheduler.create_cold_observable(
                on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))),
            on_next(260, scheduler.create_cold_observable(
                on_next(20, 4), on_next(70, 5), on_completed(200))),
            on_next(270, scheduler.create_cold_observable(
                on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))),
            on_next(320, scheduler.create_cold_observable(
                on_next(210, 9), on_next(240, 10), on_completed(300))),
            on_completed(400))

        def create():
            return xs.pipe(ops.merge(max_concurrent=2))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(260, 1), on_next(280, 4), on_next(310, 2), on_next(330, 3),
            on_next(330, 5), on_next(360, 6), on_next(440, 7), on_next(460, 8),
            on_next(670, 9), on_next(700, 10), on_completed(760)]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_mergeconcat_basic_long(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, scheduler.create_cold_observable(
                on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))),
            on_next(260, scheduler.create_cold_observable(
                on_next(20, 4), on_next(70, 5), on_completed(300))),
            on_next(270, scheduler.create_cold_observable(
                on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))),
            on_next(320, scheduler.create_cold_observable(
                on_next(210, 9), on_next(240, 10), on_completed(300))),
            on_completed(400))

        def create():
            return xs.pipe(ops.merge(max_concurrent=2))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(260, 1), on_next(280, 4), on_next(310, 2),
            on_next(330, 3), on_next(330, 5), on_next(360, 6),
            on_next(440, 7), on_next(460, 8), on_next(690, 9),
            on_next(720, 10), on_completed(780)]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_mergeconcat_basic_wide(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, scheduler.create_cold_observable(
                on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))),
            on_next(260, scheduler.create_cold_observable(
                on_next(20, 4), on_next(70, 5), on_completed(300))),
            on_next(270, scheduler.create_cold_observable(
                on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))),
            on_next(420, scheduler.create_cold_observable(
                on_next(210, 9), on_next(240, 10), on_completed(300))),
            on_completed(450))

        def create():
            return xs.pipe(ops.merge(max_concurrent=3))

        results = scheduler.start(create)

        assert results.messages == [on_next(260, 1), on_next(280, 4), on_next(280, 6), on_next(310, 2), on_next(
            330, 3), on_next(330, 5), on_next(360, 7), on_next(380, 8), on_next(630, 9), on_next(660, 10), on_completed(720)]
        assert xs.subscriptions == [subscribe(200, 450)]

    def test_mergeconcat_basic_late(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, scheduler.create_cold_observable(on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))), on_next(260, scheduler.create_cold_observable(on_next(20, 4), on_next(70, 5), on_completed(
            300))), on_next(270, scheduler.create_cold_observable(on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))), on_next(420, scheduler.create_cold_observable(on_next(210, 9), on_next(240, 10), on_completed(300))), on_completed(750))

        def create():
            return xs.pipe(ops.merge(max_concurrent=3))

        results = scheduler.start(create)

        assert results.messages == [on_next(260, 1), on_next(280, 4), on_next(280, 6), on_next(310, 2), on_next(
            330, 3), on_next(330, 5), on_next(360, 7), on_next(380, 8), on_next(630, 9), on_next(660, 10), on_completed(750)]
        assert xs.subscriptions == [subscribe(200, 750)]

    def test_mergeconcat_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, scheduler.create_cold_observable(
                on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))),
            on_next(260, scheduler.create_cold_observable(
                on_next(20, 4), on_next(70, 5), on_completed(200))),
            on_next(270, scheduler.create_cold_observable(
                on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))),
            on_next(320, scheduler.create_cold_observable(
                on_next(210, 9), on_next(240, 10), on_completed(300))),
            on_completed(400)
            )

        def create():
            return xs.pipe(ops.merge(max_concurrent=2))

        results = scheduler.start(create, disposed=450)
        assert results.messages == [
            on_next(260, 1), on_next(280, 4), on_next(310, 2),
            on_next(330, 3), on_next(330, 5), on_next(360, 6),
            on_next(440, 7)]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_mergeconcat_outererror(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, scheduler.create_cold_observable(on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))), on_next(260, scheduler.create_cold_observable(on_next(20, 4), on_next(70, 5), on_completed(
            200))), on_next(270, scheduler.create_cold_observable(on_next(10, 6), on_next(90, 7), on_next(110, 8), on_completed(130))), on_next(320, scheduler.create_cold_observable(on_next(210, 9), on_next(240, 10), on_completed(300))), on_error(400, ex))

        def create():
            return xs.pipe(ops.merge(max_concurrent=2))
        results = scheduler.start(create)

        assert results.messages == [on_next(260, 1), on_next(280, 4), on_next(
            310, 2), on_next(330, 3), on_next(330, 5), on_next(360, 6), on_error(400, ex)]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_mergeconcat_innererror(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, scheduler.create_cold_observable(
                on_next(50, 1), on_next(100, 2), on_next(120, 3), on_completed(140))),
            on_next(260, scheduler.create_cold_observable(
                on_next(20, 4), on_next(70, 5), on_completed(200))),
            on_next(270, scheduler.create_cold_observable(
                on_next(10, 6), on_next(90, 7), on_next(110, 8), on_error(140, ex))),
            on_next(320, scheduler.create_cold_observable(
                on_next(210, 9), on_next(240, 10), on_completed(300))),
            on_completed(400)
            )

        def create():
            return xs.pipe(ops.merge(max_concurrent=2))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(260, 1), on_next(280, 4), on_next(310, 2),
            on_next(330, 3), on_next(330, 5), on_next(360, 6),
            on_next(440, 7), on_next(460, 8), on_error(490, ex)]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_merge_112233(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(250, 1), on_next(300, 2), on_next(350, 3), on_completed(360))
        ys = scheduler.create_hot_observable(
            on_next(250, 1), on_next(300, 2), on_next(320, 3), on_completed(340))

        def create():
            return xs.pipe(ops.merge(ys))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(250, 1),
            on_next(250, 1),
            on_next(300, 2),
            on_next(300, 2),
            on_next(320, 3),
            on_next(350, 3),
            on_completed(360)]
        assert xs.subscriptions == [subscribe(200, 360)]



from rx.core.typing import Observer
from rx.testing import TestScheduler, ReactiveTest
from rx.core.notification import OnNext, OnError, OnCompleted

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


def test_on_next_ctor_and_props():
    n = OnNext(42)
    assert 'N' == n.kind
    assert n.has_value
    assert 42 == n.value
    assert not hasattr(n, "exception")


def test_on_next_equality():
    n1 = OnNext(42)
    n2 = OnNext(42)
    n3 = OnNext(24)
    n4 = OnCompleted()
    assert n1.equals(n1)
    assert n1.equals(n2)
    assert n2.equals(n1)
    assert not n1.equals(None)
    assert not n1.equals(n3)
    assert not n3.equals(n1)
    assert not n1.equals(n4)
    assert not n4.equals(n1)


def test_on_next_tostring():
    n1 = OnNext(42)
    assert "OnNext" in str(n1)
    assert "42" in str(n1)


class CheckOnNextObserver(Observer):
    def __init__(self):
        super(CheckOnNextObserver, self).__init__()

        self.value = None

    def on_next(self, value):
        self.value = value
        return self.value

    def on_error(self, error):
        raise NotImplementedError

    def on_completed(self):
        def func():
            raise NotImplementedError
        return func


def test_on_next_accept_observer():
    con = CheckOnNextObserver()
    n1 = OnNext(42)
    n1.accept(con)
    assert con.value == 42


class AcceptObserver(Observer):
    def __init__(self, on_next, on_error, on_completed):
        self._on_next = on_next
        self._on_error = on_error
        self._on_completed = on_completed

    def on_next(self, value):
        return self._on_next(value)

    def on_error(self, exception):
        return self._on_error(exception)

    def on_completed(self):
        return self._on_completed()


def test_on_next_accept_observer_with_result():
    n1 = OnNext(42)

    def on_next(x):
        return "OK"
    def on_error(err):
        assert False
    def on_completed():
        assert False

    res = n1.accept(AcceptObserver(on_next, on_error, on_completed))
    assert 'OK' == res


def test_on_next_accept_action():
    obs = [False]
    n1 = OnNext(42)
    def on_next(x):
        obs[0] = True
        return obs[0]
    def on_error(err):
        assert False
    def on_completed():
        assert False
    n1.accept(on_next, on_error, on_completed)

    assert obs[0]


def test_on_next_accept_action_with_result():
    n1 = OnNext(42)

    def on_next(x):
        return "OK"
    def on_error(err):
        assert False
    def on_completed():
        assert False

    res = n1.accept(on_next, on_error, on_completed)
    assert 'OK' == res


def test_throw_ctor_and_props():
    e = 'e'
    n = OnError(e)
    assert 'E'== n.kind
    assert not n.has_value
    assert e == n.exception


def test_throw_equality():
    ex1 = 'ex1'
    ex2 = 'ex2'
    n1 = OnError(ex1)
    n2 = OnError(ex1)
    n3 = OnError(ex2)
    n4 = OnCompleted()
    assert n1.equals(n1)
    assert n1.equals(n2)
    assert n2.equals(n1)
    assert not n1.equals(None)
    assert not n1.equals(n3)
    assert not n3.equals(n1)
    assert not n1.equals(n4)
    assert not n4.equals(n1)


def test_throw_tostring():
    ex = 'ex'
    n1 = OnError(ex)
    assert "OnError" in str(n1)
    assert "ex" in str(n1)


class CheckOnErrorObserver(Observer):
    def __init__(self):
        super(CheckOnErrorObserver, self).__init__()

        self.error = None

    def on_next(value):
        raise NotImplementedError()

    def on_error(self, exception):
        self.error = exception

    def on_completed(self):
        raise NotImplementedError()


def test_throw_accept_observer():
    ex = 'ex'
    obs = CheckOnErrorObserver()
    n1 = OnError(ex)
    n1.accept(obs)
    assert ex == obs.error


def test_throw_accept_observer_with_result():
    ex = 'ex'
    n1 = OnError(ex)

    def on_next(x):
        assert False
        return None
    def on_error(ex):
        return "OK"

    def on_completed():
        assert False
        return None

    res = n1.accept(AcceptObserver(on_next, on_error, on_completed))
    assert 'OK' == res


def test_throw_accept_action():
    ex = 'ex'
    obs = [False]
    n1 = OnError(ex)

    def on_next(x):
        assert False
        return None
    def on_error(ex):
        obs[0] = True
        return obs[0]
    def on_completed():
        assert False
        return None

    n1.accept(on_next, on_error, on_completed)
    assert obs[0]


def test_throw_accept_action_with_result():
    ex = 'ex'
    n1 = OnError(ex)

    def on_next(x):
        assert False
        return None
    def on_error(ex):
        return "OK"
    def on_completed():
        assert False
        return None

    res = n1.accept(on_next, on_error, on_completed)
    assert 'OK' == res


def test_close_ctor_and_props():
    n = OnCompleted()
    assert 'C' == n.kind
    assert not n.has_value
    assert not hasattr(n, "exception")


def test_close_equality():
    n1 = OnCompleted()
    n2 = OnCompleted()
    n3 = OnNext(2)
    assert n1.equals(n1)
    assert n1.equals(n2)
    assert n2.equals(n1)
    assert not n1.equals(None)
    assert not n1.equals(n3)
    assert not n3.equals(n1)


def test_close_tostring():
    n1 = OnCompleted()
    assert 'OnCompleted' in str(n1)


class CheckOnCompletedObserver(Observer):
    def __init__(self):
        super(CheckOnCompletedObserver, self).__init__()

        self.completed = False

    def on_next(self):
        raise NotImplementedError()

    def on_error(self):
        raise NotImplementedError()

    def on_completed(self):
        self.completed = True


def test_close_accept_observer():
    obs = CheckOnCompletedObserver()
    n1 = OnCompleted()
    n1.accept(obs)
    assert obs.completed


def test_close_accept_observer_with_result():
    n1 = OnCompleted()

    def on_next(x):
        assert False
        return None
    def on_error(err):
        assert False
        return None
    def on_completed():
        return "OK"

    res = n1.accept(AcceptObserver(on_next, on_error, on_completed))
    assert 'OK' == res


def test_close_accept_action():
    obs = [False]
    n1 = OnCompleted()

    def on_next(x):
        assert False
        return None
    def on_error(ex):
        assert False
        return None
    def on_completed():
        obs[0] = True
        return obs[0]

    n1.accept(on_next, on_error, on_completed)
    assert obs[0]


def test_close_accept_action_with_result():
    n1 = OnCompleted()

    def on_next(x):
        assert False
        return None
    def on_error(ex):
        assert False
        return None
    def on_completed():
        return "OK"

    res = n1.accept(on_next, on_error, on_completed)
    assert 'OK' == res


def test_to_observable_empty():
    scheduler = TestScheduler()

    def create():
        return OnCompleted().to_observable(scheduler)

    res = scheduler.start(create)
    assert res.messages == [ReactiveTest.on_completed(200)]


def test_to_observable_return():
    scheduler = TestScheduler()

    def create():
        return OnNext(42).to_observable(scheduler)

    res = scheduler.start(create)
    assert res.messages == [ReactiveTest.on_next(200, 42), ReactiveTest.on_completed(200)]


def test_to_observable_on_error():
    ex = 'ex'
    scheduler = TestScheduler()

    def create():
        return OnError(ex).to_observable(scheduler)

    res = scheduler.start(create)
    assert res.messages == [ReactiveTest.on_error(200, ex)]



from rx.testing import ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)



# test("Rx.Observable.catchException() does not lose subscription to underlying observable", 12, function () {
#     var subscribes = 0,
#             unsubscribes = 0,
#             tracer = Rx.Observable.create(function (observer) { ++subscribes return function () { ++unsubscribes } ,
#             s

#     // Try it without catchException()
#     s = tracer.subscribe()
#     strictEqual(subscribes, 1, "1 subscribes")
#     strictEqual(unsubscribes, 0, "0 unsubscribes")
#     s.dispose()
#     strictEqual(subscribes, 1, "After dispose: 1 subscribes")
#     strictEqual(unsubscribes, 1, "After dispose: 1 unsubscribes")

#     // Now try again with catchException(Observable):
#     subscribes = unsubscribes = 0
#     s = tracer.catchException(Rx.rx.never()).subscribe()
#     strictEqual(subscribes, 1, "catchException(Observable): 1 subscribes")
#     strictEqual(unsubscribes, 0, "catchException(Observable): 0 unsubscribes")
#     s.dispose()
#     strictEqual(subscribes, 1, "catchException(Observable): After dispose: 1 subscribes")
#     strictEqual(unsubscribes, 1, "catchException(Observable): After dispose: 1 unsubscribes")

#     // And now try again with catchException(function()):
#     subscribes = unsubscribes = 0
#     s = tracer.catchException(function () { return Rx.rx.never() .subscribe()
#     strictEqual(subscribes, 1, "catchException(function): 1 subscribes")
#     strictEqual(unsubscribes, 0, "catchException(function): 0 unsubscribes")
#     s.dispose()
#     strictEqual(subscribes, 1, "catchException(function): After dispose: 1 subscribes")
#     strictEqual(unsubscribes, 1, "catchException(function): After dispose: 1 unsubscribes") // this one FAILS (unsubscribes is 0)
#

if __name__ == '__main__':
    test_combine_latest_return_empty()



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestOf(unittest.TestCase):
    def test_of(self):
        results = []

        rx.of(1, 2, 3, 4, 5).subscribe(results.append)

        assert(str([1, 2, 3, 4, 5]) == str(results))

    def test_of_empty(self):
        results = []

        rx.of().subscribe(results.append)

        assert(len(results) == 0)

    def teest_of_with_scheduler(self):
        scheduler = TestScheduler()

        def create():
            return rx.of(1, 2, 3, 4, 5)

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(201, 1),
            on_next(202, 2),
            on_next(203, 3),
            on_next(204, 4),
            on_next(205, 5),
            on_completed(206)]

    def teest_of_with_scheduler_empty(self):
        scheduler = TestScheduler()

        def create():
            return rx.of(scheduler=scheduler)

        results = scheduler.start(create=create)

        assert results.messages == [
            on_completed(201)]



import unittest

from rx import return_value, throw, empty, create
from rx.testing import TestScheduler, ReactiveTest
from rx.disposable import SerialDisposable

from rx import operators as ops

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass

# Helper function for raising exceptions within lambdas


def _raise(ex):
    raise RxException(ex)


class TestSelect(unittest.TestCase):

    def test_starmap_never(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable()

        invoked = [0]

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                return x + y

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == []
        assert xs.subscriptions == [ReactiveTest.subscribe(200, 1000)]
        assert invoked[0] == 0

    def test_starmap_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            # 200 subscribe
            on_completed(300),
            )

        invoked = [0]

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                return x + y

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [on_completed(300)]
        assert xs.subscriptions == [ReactiveTest.subscribe(200, 300)]
        assert invoked[0] == 0

    def test_starmap_subscription_error(self):
        mapper = ops.starmap(lambda x, y: (x, y))

        with self.assertRaises(RxException):
            return_value((1, 10)).pipe(
                mapper
            ).subscribe(lambda x: _raise("ex"))

        with self.assertRaises(RxException):
            throw('ex').pipe(
                mapper
            ).subscribe(on_error=lambda ex: _raise(ex))

        with self.assertRaises(RxException):
            empty().pipe(
                mapper
            ).subscribe(lambda x: x, lambda ex: ex, lambda: _raise('ex'))

        def subscribe(observer, scheduler=None):
            _raise('ex')

        with self.assertRaises(RxException):
            create(subscribe).pipe(
                mapper
            ).subscribe()

    def test_starmap_dispose_inside_mapper(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(110, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(310, (3, 30)),
            on_next(410, (4, 40)))

        results = scheduler.create_observer()
        d = SerialDisposable()
        invoked = [0]

        def mapper(x, y):
            invoked[0] += 1
            if scheduler._clock > 250:
                d.dispose()
            return x + y

        d.disposable = xs.pipe(
            ops.starmap(mapper)
        ).subscribe(results, scheduler)

        def action(scheduler, state):
            return d.dispose()

        scheduler.schedule_absolute(ReactiveTest.disposed, action)
        scheduler.start()

        assert results.messages == [
                on_next(110, 11),
                on_next(210, 22)]

        assert xs.subscriptions == [ReactiveTest.subscribe(0, 310)]
        assert invoked[0] == 3

    def test_starmap_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_completed(400),
            on_next(410, (-1, -10)),
            on_completed(420),
            on_error(430, 'ex'))

        invoked = [0]

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                return x + y

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 22),
            on_next(240, 33),
            on_next(290, 44),
            on_next(350, 55),
            on_completed(400)]

        assert xs.subscriptions == [ReactiveTest.subscribe(200, 400)]
        assert invoked[0] == 4

    def test_starmap_not_completed(self):
        scheduler = TestScheduler()
        invoked = [0]
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)))

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                return x + y

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 22),
            on_next(240, 33),
            on_next(290, 44),
            on_next(350, 55)]

        assert xs.subscriptions == [subscribe(200, 1000)]
        assert invoked[0] == 4

    def test_starmap_no_mapper(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_completed(400),
            on_next(410, (-1, -10)),
            on_completed(420),
            on_error(430, 'ex'))

        def factory():
            return xs.pipe(ops.starmap())

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_completed(400)]

        assert xs.subscriptions == [ReactiveTest.subscribe(200, 400)]


    def test_starmap_mapper_with_one_element(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1,)),
            # 200 subscribe
            on_next(210, (2,)),
            on_next(240, (3,)),
            on_next(290, (4,)),
            on_next(350, (5,)),
            on_completed(400),
            on_next(410, (-1,)),
            on_completed(420),
            on_error(430, 'ex'))

        invoked = [0]

        def factory():
            def mapper(x):
                invoked[0] += 1
                return x * 10

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 20),
            on_next(240, 30),
            on_next(290, 40),
            on_next(350, 50),
            on_completed(400)]

        assert xs.subscriptions == [ReactiveTest.subscribe(200, 400)]
        assert invoked[0] == 4

    def test_starmap_mapper_with_three_elements(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10, 100)),
            # 200 subscribe
            on_next(210, (2, 20, 200)),
            on_next(240, (3, 30, 300)),
            on_next(290, (4, 40, 400)),
            on_next(350, (5, 50, 500)),
            on_completed(400),
            on_next(410, (-1, -10, -100)),
            on_completed(420),
            on_error(430, 'ex'))

        invoked = [0]

        def factory():
            def mapper(x, y, z):
                invoked[0] += 1
                return x + y + z

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 222),
            on_next(240, 333),
            on_next(290, 444),
            on_next(350, 555),
            on_completed(400)]

        assert xs.subscriptions == [ReactiveTest.subscribe(200, 400)]
        assert invoked[0] == 4

    def test_starmap_mapper_with_args(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_completed(400),
            on_next(410, (-1, -10)),
            on_completed(420),
            on_error(430, 'ex'))

        invoked = [0]

        def factory():
            def mapper(*args):
                invoked[0] += 1
                return sum(args)

            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 22),
            on_next(240, 33),
            on_next(290, 44),
            on_next(350, 55),
            on_completed(400)]

        assert xs.subscriptions == [ReactiveTest.subscribe(200, 400)]
        assert invoked[0] == 4

    def test_starmap_error(self):
        scheduler = TestScheduler()
        ex = 'ex'
        invoked = [0]
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_error(400, ex),
            on_next(410, (-1, -10)),
            on_completed(420),
            on_error(430, ex))

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                return x + y
            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 22),
            on_next(240, 33),
            on_next(290, 44),
            on_next(350, 55),
            on_error(400, ex)]

        assert xs.subscriptions == [subscribe(200, 400)]
        assert invoked[0] == 4

    def test_starmap_mapper_error(self):
        scheduler = TestScheduler()
        invoked = [0]
        ex = 'ex'
        xs = scheduler.create_hot_observable(
            # 100 create
            on_next(180, (1, 10)),
            # 200 subscribe
            on_next(210, (2, 20)),
            on_next(240, (3, 30)),
            on_next(290, (4, 40)),
            on_next(350, (5, 50)),
            on_completed(400),
            on_next(410, (-1, -10)),
            on_completed(420),
            on_error(430, ex))

        def factory():
            def mapper(x, y):
                invoked[0] += 1
                if invoked[0] == 3:
                    raise Exception(ex)

                return x + y
            return xs.pipe(ops.starmap(mapper))

        results = scheduler.start(factory)
        assert results.messages == [
            on_next(210, 22),
            on_next(240, 33),
            on_error(290, ex)]

        assert xs.subscriptions == [subscribe(200, 290)]
        assert invoked[0] == 3


if __name__ == '__main__':
    unittest.main()



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestExpand(unittest.TestCase):

    def test_expand_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_completed(300))

        def create():
            def mapper():
                return scheduler.create_cold_observable(on_next(100, 1), on_next(200, 2), on_completed(300))

            return xs.pipe(ops.expand(mapper))
        results = scheduler.start(create)

        assert results.messages == [on_completed(300)]
        assert xs.subscriptions == [subscribe(200, 300)]

    def test_expand_error(self):
        scheduler = TestScheduler()
        ex = 'ex'
        xs = scheduler.create_hot_observable(on_error(300, ex))

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_next(100 + x, 2 * x), on_next(200 + x, 3 * x), on_completed(300 + x))
            return xs.pipe(ops.expand(mapper))
        results = scheduler.start(create)

        assert results.messages == [on_error(300, ex)]
        assert xs.subscriptions == [subscribe(200, 300)]

    def test_expand_never(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable()

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_next(100 + x, 2 * x), on_next(200 + x, 3 * x), on_completed(300 + x))
            return xs.pipe(ops.expand(mapper))

        results = scheduler.start(create)

        assert results.messages == []
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_expand_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(550, 1), on_next(850, 2), on_completed(950))

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_next(100, 2 * x), on_next(200, 3 * x), on_completed(300))
            return xs.pipe(ops.expand(mapper))
        results = scheduler.start(create)

        assert results.messages == [on_next(550, 1), on_next(650, 2), on_next(750, 3), on_next(750, 4), on_next(850, 2), on_next(850, 6), on_next(850, 6), on_next(850, 8), on_next(950, 9), on_next(950, 12), on_next(950, 4), on_next(950, 12), on_next(950, 12), on_next(950, 16)]
        assert xs.subscriptions == [subscribe(200, 950)]

    def test_expand_on_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(550, 1), on_next(850, 2), on_completed(950))

        def create():
            def mapper(x):
                raise Exception(ex)
            return xs.pipe(ops.expand(mapper))
        results = scheduler.start(create)

        assert results.messages == [on_next(550, 1), on_error(550, ex)]
        assert xs.subscriptions == [subscribe(200, 550)]



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestTakeLastBuffer(unittest.TestCase):

    def test_take_last_buffer_zero_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.take_last_buffer(0))

        res = scheduler.start(create)

        def predicate(lst):
            return len(lst) == 0

        assert [on_next(650, predicate), on_completed(650)] == res.messages
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_take_last_buffer_zero_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_error(650, ex))

        def create():
            return xs.pipe(ops.take_last_buffer(0))

        res = scheduler.start(create)

        assert res.messages == [on_error(650, ex)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_take_last_buffer_zero_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9))

        def create():
            return xs.pipe(ops.take_last_buffer(0))

        res = scheduler.start(create)

        assert res.messages == []
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_take_last_buffer_one_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.take_last_buffer(1))

        res = scheduler.start(create)

        def predicate(lst):
            return lst == [9]

        assert [on_next(650, predicate), on_completed(650)] == res.messages
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_take_last_buffer_one_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_error(650, ex))

        def create():
            return xs.pipe(ops.take_last_buffer(1))

        res = scheduler.start(create)

        assert res.messages == [on_error(650, ex)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_take_last_buffer_one_disposed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9))

        def create():
            return xs.pipe(ops.take_last_buffer(1))

        res = scheduler.start(create)

        assert res.messages == []
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_take_last_buffer_three_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.take_last_buffer(3))

        res = scheduler.start(create)

        def predicate(lst):
            return lst == [7, 8, 9]

        assert [on_next(650, predicate), on_completed(650)] == res.messages
        assert xs.subscriptions == [subscribe(200, 650)]

# def test_Take_last_buffer_Three_Error():
#     var ex, res, scheduler, xs
#     ex = 'ex'
#     scheduler = TestScheduler()
#     xs = scheduler.create_hot_observable(on_next(180, 1), on_next(210, 2), on_next(250, 3), on_next(270, 4), on_next(310, 5), on_next(360, 6), on_next(380, 7), on_next(410, 8), on_next(590, 9), on_error(650, ex))
#     res = scheduler.start(create)
#         return xs.pipe(ops.take_last_buffer(3))

#     assert res.messages == [on_error(650, ex)]
#     assert xs.subscriptions == [subscribe(200, 650)]

# def test_Take_last_buffer_Three_Disposed():
#     var res, scheduler, xs
#     scheduler = TestScheduler()
#     xs = scheduler.create_hot_observable(on_next(180, 1), on_next(210, 2), on_next(250, 3), on_next(270, 4), on_next(310, 5), on_next(360, 6), on_next(380, 7), on_next(410, 8), on_next(590, 9))
#     res = scheduler.start(create)
#         return xs.pipe(ops.take_last_buffer(3))

#     assert res.messages == []
#     assert xs.subscriptions == [subscribe(200, 1000)]




import unittest

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created



class TestZip(unittest.TestCase):

    def test_zip_never_never(self):
        scheduler = TestScheduler()
        o1 = rx.never()
        o2 = rx.never()

        def create():
            return o1.pipe(ops.zip(o2))

        results = scheduler.start(create)
        assert results.messages == []

    def test_zip_never_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_completed(210)]
        o1 = rx.never()
        o2 = scheduler.create_hot_observable(msgs)

        def create():
            return o1.pipe(ops.zip(o2))

        results = scheduler.start(create)
        assert results.messages == []

    def test_zip_empty_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_completed(210)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_completed(210)]

    def test_zip_empty_non_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_completed(215)]

    def test_zip_non_empty_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_completed(215)]

    def test_zip_never_non_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs)
        e2 = rx.never()

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == []

    def test_zip_non_empty_never(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs)
        e2 = rx.never()

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == []

    def test_zip_non_empty_non_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_next(220, 3), on_completed(240)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_next(220, 2 + 3), on_completed(240)]

    def test_zip_empty_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_error_empty(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_never_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = rx.never()
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_error_never(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = rx.never()
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_error_error(self):
        ex1 = 'ex1'
        ex2 = 'ex2'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_error(230, ex1)]
        msgs2 = [on_next(150, 1), on_error(220, ex2)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex2)]

    def test_zip_some_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_error_some(self):
        ex = 'ex'
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.zip(e1),
                ops.map(sum))

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_zip_some_data_asymmetric1(self):
        scheduler = TestScheduler()

        def msgs1_factory():
            results = []
            for i in range(5):
                results.append(on_next(205 + i * 5, i))
            return results
        msgs1 = msgs1_factory()

        def msgs2_factory():
            results = []
            for i in range(10):
                results.append(on_next(205 + i * 8, i))
            return results
        msgs2 = msgs2_factory()

        length = min(len(msgs1), len(msgs2))
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create).messages
        assert(length == len(results))
        for i in range(length):
            _sum = msgs1[i].value.value + msgs2[i].value.value
            time = max(msgs1[i].time, msgs2[i].time)
            assert(results[i].value.kind == 'N'
                   and results[i].time == time
                   and results[i].value.value == _sum)

    def test_zip_some_data_asymmetric2(self):
        scheduler = TestScheduler()

        def msgs1_factory():
            results = []
            for i in range(10):
                results.append(on_next(205 + i * 5, i))

            return results
        msgs1 = msgs1_factory()

        def msgs2_factory():
            results = []
            for i in range(5):
                results.append(on_next(205 + i * 8, i))
            return results
        msgs2 = msgs2_factory()

        length = min(len(msgs1), len(msgs2))
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create).messages
        assert(length == len(results))
        for i in range(length):
            _sum = msgs1[i].value.value + msgs2[i].value.value
            time = max(msgs1[i].time, msgs2[i].time)
            assert(results[i].value.kind == 'N'
                   and results[i].time == time
                   and results[i].value.value == _sum)

    def test_zip_some_data_symmetric(self):
        scheduler = TestScheduler()

        def msgs1_factory():
            results = []
            for i in range(10):
                results.append(on_next(205 + i * 5, i))
            return results
        msgs1 = msgs1_factory()

        def msgs2_factory():
            results = []
            for i in range(10):
                results.append(on_next(205 + i * 8, i))
            return results
        msgs2 = msgs2_factory()

        length = min(len(msgs1), len(msgs2))
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.zip(e2),
                ops.map(sum))

        results = scheduler.start(create).messages
        assert(length == len(results))
        for i in range(length):
            _sum = msgs1[i].value.value + msgs2[i].value.value
            time = max(msgs1[i].time, msgs2[i].time)
            assert(results[i].value.kind == 'N'
                   and results[i].time == time
                   and results[i].value.value == _sum)

    def test_zip_with_iterable_never_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1))
        n2 = []

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == []
        assert n1.subscriptions == [subscribe(200, 1000)]

    def test_zip_with_iterable_empty_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1), on_completed(210))
        n2 = []

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_completed(210)]
        assert n1.subscriptions == [subscribe(200, 210)]

    def test_zip_with_iterable_empty_non_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1), on_completed(210))
        n2 = [2]

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_completed(210)]
        assert n1.subscriptions == [subscribe(200, 210)]

    def test_zip_with_iterable_non_empty_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(
            on_next(150, 1), on_next(215, 2), on_completed(220))
        n2 = []

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_completed(215)]
        assert n1.subscriptions == [subscribe(200, 215)]

    def test_zip_with_iterable_never_non_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1))
        n2 = [2]

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == []
        assert n1.subscriptions == [subscribe(200, 1000)]

    def test_zip_with_iterable_non_empty_non_empty(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(
                on_next(150, 1), on_next(215, 2), on_completed(230))
        n2 = [3]

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_next(215, 2 + 3), on_completed(230)]
        assert n1.subscriptions == [subscribe(200, 230)]

    def test_zip_with_iterable_error_empty(self):
        ex = 'ex'
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1), on_error(220, ex))
        n2 = []

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_error(220, ex)]
        assert n1.subscriptions == [subscribe(200, 220)]

    def test_zip_with_iterable_error_some(self):
        ex = 'ex'
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(on_next(150, 1), on_error(220, ex))
        n2 = [2]

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_error(220, ex)]
        assert n1.subscriptions == [subscribe(200, 220)]

    def test_zip_with_iterable_some_data_both_sides(self):
        scheduler = TestScheduler()
        n1 = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_next(220, 3),
            on_next(230, 4), on_next(240, 5))
        n2 = [5, 4, 3, 2]

        def create():
            return n1.pipe(
                ops.zip_with_iterable(n2),
                ops.map(sum))

        results = scheduler.start(create)

        assert results.messages == [on_next(210, 7), on_next(220, 7),
                                    on_next(230, 7), on_next(240, 7)]
        assert n1.subscriptions == [subscribe(200, 1000)]



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest, MockDisposable

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestUsing(unittest.TestCase):
    def test_using_null(self):
        disp = [None]
        xs = [None]
        _d = [None]

        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]

        def create():
            def create_resources():
                dispose_invoked[0] += 1
                disp[0] = None
                return disp[0]

            def create_observable(d):
                _d[0] = d
                create_invoked[0] += 1
                xs[0] = scheduler.create_cold_observable(
                        on_next(100, scheduler.clock), on_completed(200))
                return xs[0]
            return rx.using(create_resources, create_observable)

        results = scheduler.start(create)

        assert (disp[0] == _d[0])
        assert results.messages == [on_next(300, 200), on_completed(400)]
        assert (1 == create_invoked[0])
        assert (1 == dispose_invoked[0])
        assert xs[0].subscriptions == [subscribe(200, 400)]
        assert (disp[0] is None)

    def test_using_complete(self):
        disp = [None]
        xs = [None]
        _d = [None]
        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]

        def create():
            def create_resource():
                dispose_invoked[0] += 1
                disp[0] = MockDisposable(scheduler)
                return disp[0]

            def create_observable(d):
                _d[0] = d
                create_invoked[0] += 1
                xs[0] = scheduler.create_cold_observable(
                        on_next(100, scheduler.clock), on_completed(200))
                return xs[0]
            return rx.using(create_resource, create_observable)

        results = scheduler.start(create)

        assert disp == _d
        assert results.messages == [on_next(300, 200), on_completed(400)]
        assert create_invoked[0] == 1
        assert dispose_invoked[0] == 1
        assert xs[0].subscriptions == [subscribe(200, 400)]
        disp[0].disposes = [200, 400]

    def test_using_error(self):
        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]
        ex = 'ex'
        disp = [None]
        xs = [None]
        _d = [None]

        def create():
            def create_resource():
                dispose_invoked[0] += 1
                disp[0] = MockDisposable(scheduler)
                return disp[0]

            def create_observable(d):
                _d[0] = d
                create_invoked[0] += 1
                xs[0] = scheduler.create_cold_observable(
                        on_next(100, scheduler.clock), on_error(200, ex))
                return xs[0]
            return rx.using(create_resource, create_observable)
        results = scheduler.start(create)

        assert disp[0] == _d[0]
        assert results.messages == [on_next(300, 200), on_error(400, ex)]
        assert create_invoked[0] == 1
        assert dispose_invoked[0] == 1
        assert xs[0].subscriptions == [subscribe(200, 400)]
        assert disp[0].disposes == [200, 400]

    def test_using_dispose(self):
        disp = [None]
        xs = [None]
        _d = [None]
        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]

        def create():
            def create_resource():
                dispose_invoked[0] += 1
                disp[0] = MockDisposable(scheduler)
                return disp[0]

            def create_observable(d):
                _d[0] = d
                create_invoked[0] += 1
                xs[0] = scheduler.create_cold_observable(
                        on_next(100, scheduler.clock),
                        on_next(1000, scheduler.clock + 1))
                return xs[0]
            return rx.using(create_resource, create_observable)
        results = scheduler.start(create)

        assert disp[0] == _d[0]
        assert results.messages == [on_next(300, 200)]
        assert 1 == create_invoked[0]
        assert 1 == dispose_invoked[0]
        assert xs[0].subscriptions == [subscribe(200, 1000)]
        assert disp[0].disposes == [200, 1000]

    def test_using_throw_resource_mapper(self):
        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]
        ex = 'ex'

        def create():
            def create_resource():
                dispose_invoked[0] += 1
                raise _raise(ex)

            def create_observable(d):
                create_invoked[0] += 1
                return rx.never()

            return rx.using(create_resource, create_observable)
        results = scheduler.start(create)

        assert results.messages == [on_error(200, ex)]
        assert(0 == create_invoked[0])
        assert(1 == dispose_invoked[0])

    def test_using_throw_resource_usage(self):
        scheduler = TestScheduler()
        dispose_invoked = [0]
        create_invoked = [0]
        disp = [None]
        ex = 'ex'

        def create():
            def create_resource():
                dispose_invoked[0] += 1
                disp[0] = MockDisposable(scheduler)
                return disp[0]

            def create_observable(d):
                create_invoked[0] += 1
                _raise(ex)

            return rx.using(create_resource, create_observable)
        results = scheduler.start(create)

        assert results.messages == [on_error(200, ex)]
        assert create_invoked[0] == 1
        assert dispose_invoked[0] == 1
        assert disp[0].disposes == [200, 200]



#!/usr/bin/env python3

import sys 

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

from configparser import ConfigParser


# General project metadata is stored in project.cfg
with open('project.cfg') as project_file:
    config = ConfigParser()
    config.read_file(project_file)
    project_meta = dict(config.items('project'))


# Populate the long_description field from README.rst
with open('README.rst') as readme_file:
    project_meta['long_description'] = readme_file.read()


needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

setup(
    **{key: project_meta[key] for key in (
        'name',
        'version',
        'description',
        'long_description',
        'author',
        'author_email',
        'license',
        'url',
        'download_url'
    )},
    zip_safe=True,
    python_requires='>=3.6.0',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    setup_requires=pytest_runner,
    tests_require=['pytest>=4.6.1', 'pytest-asyncio>=0.10.0', 'coverage>=4.5.3'],

    package_data={'rx': ['py.typed']},
    packages=['rx', 'rx.internal', 'rx.core', 'rx.core.abc',
              'rx.core.operators', 'rx.core.operators.connectable',
              'rx.core.observable', 'rx.core.observer',
              'rx.scheduler', 'rx.scheduler.eventloop', 'rx.scheduler.mainloop',
              'rx.operators', 'rx.disposable', 'rx.subject',
              'rx.testing'],
    package_dir={'rx': 'rx'},
    include_package_data=True
)



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSkipLast(unittest.TestCase):
    def test_skip_last_zero_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.skip_last(0))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(210, 2), on_next(250, 3), on_next(270, 4),
                on_next(310, 5), on_next(360, 6), on_next(380, 7),
                on_next(410, 8), on_next(590, 9), on_completed(650)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_zero_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_error(650, ex))

        def create():
            return xs.pipe(ops.skip_last(0))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(210, 2), on_next(250, 3), on_next(270, 4),
                on_next(310, 5), on_next(360, 6), on_next(380, 7),
                on_next(410, 8), on_next(590, 9), on_error(650, ex)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_zero_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9))

        def create():
            return xs.pipe(ops.skip_last(0))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(210, 2), on_next(250, 3), on_next(270, 4),
                on_next(310, 5), on_next(360, 6), on_next(380, 7),
                on_next(410, 8), on_next(590, 9)]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_skip_last_one_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.skip_last(1))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(250, 2), on_next(270, 3), on_next(310, 4),
                on_next(360, 5), on_next(380, 6), on_next(410, 7),
                on_next(590, 8), on_completed(650)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_one_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_error(650, ex))

        def create():
            return xs.pipe(ops.skip_last(1))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(250, 2), on_next(270, 3), on_next(310, 4),
                on_next(360, 5), on_next(380, 6), on_next(410, 7),
                on_next(590, 8), on_error(650, ex)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_one_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9))

        def create():
            return xs.pipe(ops.skip_last(1))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(250, 2), on_next(270, 3), on_next(310, 4),
                on_next(360, 5), on_next(380, 6), on_next(410, 7),
                on_next(590, 8)]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_skip_last_three_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_completed(650))

        def create():
            return xs.pipe(ops.skip_last(3))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(310, 2), on_next(360, 3), on_next(380, 4),
                on_next(410, 5), on_next(590, 6), on_completed(650)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_three_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9),
                on_error(650, ex))

        def create():
            return xs.pipe(ops.skip_last(3))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(310, 2), on_next(360, 3), on_next(380, 4),
                on_next(410, 5), on_next(590, 6), on_error(650, ex)]
        assert xs.subscriptions == [subscribe(200, 650)]

    def test_skip_last_three_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(180, 1), on_next(210, 2), on_next(250, 3),
                on_next(270, 4), on_next(310, 5), on_next(360, 6),
                on_next(380, 7), on_next(410, 8), on_next(590, 9))

        def create():
            return xs.pipe(ops.skip_last(3))

        results = scheduler.start(create)

        assert results.messages == [
                on_next(310, 2), on_next(360, 3), on_next(380, 4),
                on_next(410, 5), on_next(590, 6)]
        assert xs.subscriptions == [subscribe(200, 1000)]



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestIf_then(unittest.TestCase):
    def test_if_true(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, 1), on_next(250, 2), on_completed(300))
        ys = scheduler.create_hot_observable(on_next(310, 3), on_next(350, 4), on_completed(400))

        def create():
            return rx.if_then(lambda: True, xs, ys)
        results = scheduler.start(create=create)

        assert results.messages == [on_next(210, 1), on_next(250, 2), on_completed(300)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == []

    def test_if_false(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, 1), on_next(250, 2), on_completed(300))
        ys = scheduler.create_hot_observable(on_next(310, 3), on_next(350, 4), on_completed(400))

        def create():
            return rx.if_then(lambda: False, xs, ys)
        results = scheduler.start(create=create)

        assert results.messages == [on_next(310, 3), on_next(350, 4), on_completed(400)]
        assert xs.subscriptions == []
        assert ys.subscriptions == [subscribe(200, 400)]

    def test_if_on_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, 1), on_next(250, 2), on_completed(300))
        ys = scheduler.create_hot_observable(on_next(310, 3), on_next(350, 4), on_completed(400))

        def create():
            def condition():
                raise Exception(ex)
            return rx.if_then(condition, xs, ys)
        results = scheduler.start(create=create)

        assert results.messages == [on_error(200, ex)]
        assert xs.subscriptions == []
        assert ys.subscriptions == []

    def test_if_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(210, 1), on_next(250, 2))
        ys = scheduler.create_hot_observable(on_next(310, 3), on_next(350, 4), on_completed(400))

        def create():
            return rx.if_then(lambda: True, xs, ys)
        results = scheduler.start(create=create)

        assert results.messages == [on_next(210, 1), on_next(250, 2)]
        assert xs.subscriptions == [subscribe(200, 1000)]
        assert ys.subscriptions == []

    def test_if_default_completed(self):
        b = [False]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(110, 1), on_next(220, 2), on_next(330, 3), on_completed(440))

        def action(scheduler, state):
            b[0] = True
        scheduler.schedule_absolute(150, action)

        def create():
            def condition():
                return b[0]
            return rx.if_then(condition, xs)
        results = scheduler.start(create)

        assert results.messages == [on_next(220, 2), on_next(330, 3), on_completed(440)]
        assert xs.subscriptions == [subscribe(200, 440)]

    def test_if_default_error(self):
        ex = 'ex'
        b = [False]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(110, 1), on_next(220, 2), on_next(330, 3), on_error(440, ex))

        def action(scheduler, state):
            b[0] = True
        scheduler.schedule_absolute(150, action)

        def create():
            def condition():
                return b[0]
            return rx.if_then(condition, xs)
        results = scheduler.start(create)

        assert results.messages == [on_next(220, 2), on_next(330, 3), on_error(440, ex)]
        assert xs.subscriptions == [subscribe(200, 440)]

    def test_if_default_never(self):
        b = [False]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(110, 1), on_next(220, 2), on_next(330, 3))

        def action(scheduler, state):
            b[0] = True
        scheduler.schedule_absolute(150, action)

        def create():
            def condition():
                return b[0]
            return rx.if_then(condition, xs)
        results = scheduler.start(create)

        assert results.messages == [on_next(220, 2), on_next(330, 3)]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_if_default_other(self):
        b = [True]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
                on_next(110, 1), on_next(220, 2), on_next(330, 3), on_error(440, 'ex'))

        def action(scheduler, state):
            b[0] = False
        scheduler.schedule_absolute(150, action)

        def create():
            def condition():
                return b[0]
            return rx.if_then(condition, xs)
        results = scheduler.start(create)

        assert results.messages == [on_completed(200)]
        assert xs.subscriptions == []



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestForIn(unittest.TestCase):

    def test_for_basic(self):
        scheduler = TestScheduler()

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(
                    on_next(x * 100 + 10, x * 10 + 1),
                    on_next(x * 100 + 20, x * 10 + 2),
                    on_next(x * 100 + 30, x * 10 + 3),
                    on_completed(x * 100 + 40))
            return rx.for_in([1, 2, 3], mapper)

        results = scheduler.start(create=create)
        assert results.messages == [
            on_next(310, 11), on_next(320, 12), on_next(330, 13),
            on_next(550, 21), on_next(560, 22), on_next(570, 23),
            on_next(890, 31), on_next(900, 32), on_next(910, 33),
            on_completed(920)
        ]

    def test_for_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            def mapper(x):
                raise Exception(ex)
            return rx.for_in([1, 2, 3], mapper)
        results = scheduler.start(create=create)
        assert results.messages == [on_error(200, ex)]



import unittest

from configparser import ConfigParser
from os.path import abspath, dirname, join
from re import match


class VersionTest(unittest.TestCase):

    def test_version(self):

        root = abspath(join(dirname(__file__), '..'))

        with open(join(root, 'project.cfg')) as project_file:
            config = ConfigParser()
            config.read_file(project_file)
            project_meta = dict(config.items('project'))

        version = None
        with open(join(root, 'rx', '__init__.py')) as init_py:
            for line in init_py:
                version = match('\\s*__version__\\s*=\\s*[\'"](.*)[\'"]', line)
                if version is not None:
                    version = version.group(1)
                    break

        assert project_meta['version'] == version



import unittest

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestFinally(unittest.TestCase):
    def test_finally_only_called_once_empty(self):
        invasserte_count = [0]

        def action():
            invasserte_count[0] += 1
            return invasserte_count
        some_observable = rx.empty().pipe(ops.finally_action(action))

        d = some_observable.subscribe()
        d.dispose()
        d.dispose()
        self.assertEqual(1, invasserte_count[0])

    def test_finally_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))
        invasserted = [False]

        def create():
            def action():
                invasserted[0] = True
                return invasserted[0]
            return xs.pipe(ops.finally_action(action))

        results = scheduler.start(create).messages
        self.assertEqual(1, len(results))
        assert(results[0].value.kind == 'C' and results[0].time == 250)
        assert(invasserted[0])

    def test_finally_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(210, 2), on_completed(250))
        invasserted = [False]

        def create():
            def action():
                invasserted[0] = True
                return invasserted[0]

            return xs.pipe(ops.finally_action(action))

        results = scheduler.start(create).messages
        self.assertEqual(2, len(results))
        assert(results[0].value.kind == 'N' and results[0].time == 210 and results[0].value.value == 2)
        assert(results[1].value.kind == 'C' and results[1].time == 250)
        assert(invasserted[0])

    def test_finally_on_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))
        invasserted = [False]

        def create():
            def action():
                invasserted[0] = True
                return invasserted[0]

            return xs.pipe(ops.finally_action(action))

        results = scheduler.start(create).messages
        self.assertEqual(1, len(results))
        assert(results[0].value.kind == 'E' and results[0].time == 250 and results[0].value.exception == ex)
        assert(invasserted[0])



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestDistinctUntilChanged(unittest.TestCase):
    def test_default_if_empty_non_empty1(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(280, 42), on_next(360, 43), on_completed(420))

        def create():
            return xs.pipe(ops.default_if_empty())

        results = scheduler.start(create)

        assert results.messages == [on_next(280, 42), on_next(360, 43), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 420)]

    def test_default_if_empty_non_empty2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(280, 42), on_next(360, 43), on_completed(420))

        def create():
            return xs.pipe(ops.default_if_empty(-1))

        results = scheduler.start(create)

        assert results.messages == [on_next(280, 42), on_next(360, 43), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 420)]

    def test_default_if_empty_empty1(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_completed(420))

        def create():
            return xs.pipe(ops.default_if_empty(None))

        results = scheduler.start(create)

        assert results.messages == [on_next(420, None), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 420)]

    def test_default_if_empty_empty2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_completed(420))

        def create():
            return xs.pipe(ops.default_if_empty(-1))
        results = scheduler.start(create)

        assert results.messages == [on_next(420, -1), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 420)]



import unittest

from rx import empty, never, throw, operators as _
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestDebounce(unittest.TestCase):
    def test_debounce_timespan_allpass(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(200, 2), on_next(250, 3), on_next(
            300, 4), on_next(350, 5), on_next(400, 6), on_next(450, 7), on_next(500, 8), on_completed(550))

        def create():
            return xs.pipe(_.debounce(40))

        results = scheduler.start(create)

        assert results.messages == [on_next(290, 3), on_next(340, 4), on_next(
            390, 5), on_next(440, 6), on_next(490, 7), on_next(540, 8), on_completed(550)]

    def test_debounce_timespan_allpass_error_end(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(200, 2), on_next(250, 3), on_next(
            300, 4), on_next(350, 5), on_next(400, 6), on_next(450, 7), on_next(500, 8), on_error(550, ex))

        def create():
            return xs.pipe(_.debounce(40))

        results = scheduler.start(create)
        assert results.messages == [on_next(290, 3), on_next(340, 4), on_next(
            390, 5), on_next(440, 6), on_next(490, 7), on_next(540, 8), on_error(550, ex)]

    def test_debounce_timespan_alldrop(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(200, 2), on_next(250, 3), on_next(
            300, 4), on_next(350, 5), on_next(400, 6), on_next(450, 7), on_next(500, 8), on_completed(550))

        def create():
            return xs.pipe(_.debounce(60))

        results = scheduler.start(create)
        assert results.messages == [on_next(550, 8), on_completed(550)]

    def test_debounce_timespan_alldrop_error_end(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(200, 2), on_next(250, 3), on_next(
            300, 4), on_next(350, 5), on_next(400, 6), on_next(450, 7), on_next(500, 8), on_error(550, ex))

        def create():
            return xs.pipe(_.debounce(60))

        results = scheduler.start(create)
        assert results.messages == [on_error(550, ex)]

    def test_debounce_timespan_some_drop(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(250, 2), on_next(350, 3), on_next(
            370, 4), on_next(421, 5), on_next(480, 6), on_next(490, 7), on_next(500, 8), on_completed(600))

        def create():
            return xs.pipe(_.debounce(50))

        results = scheduler.start(create)
        assert results.messages == [on_next(300, 2), on_next(
            420, 4), on_next(471, 5), on_next(550, 8), on_completed(600)]

    def test_debounce_empty(self):
        scheduler = TestScheduler()

        def create():
            return empty().pipe(_.debounce(10))

        results = scheduler.start(create)

        assert results.messages == [on_completed(200)]

    def test_debounce_error(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return throw(ex).pipe(_.debounce(10))

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_debounce_never(self):
        scheduler = TestScheduler()

        def create():
            return never().pipe(_.debounce(10))

        results = scheduler.start(create)
        assert results.messages == []

    def test_debounce_duration_delay_behavior(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, -1),
            on_next(250, 0),
            on_next(280, 1),
            on_next(310, 2),
            on_next(350, 3),
            on_next(400, 4),
            on_completed(550))
        ys = [scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(
            on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99))]

        def create():
            def mapper(x):
                return ys[x]

            return xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 20, 0), on_next(280 + 20, 1), on_next(310 + 20, 2),
                                    on_next(350 + 20, 3), on_next(400 + 20, 4), on_completed(550)]
        assert xs.subscriptions == [subscribe(200, 550)]
        assert ys[0].subscriptions == [subscribe(250, 250 + 20)]
        assert ys[1].subscriptions == [subscribe(280, 280 + 20)]
        assert ys[2].subscriptions == [subscribe(310, 310 + 20)]
        assert ys[3].subscriptions == [subscribe(350, 350 + 20)]
        assert ys[4].subscriptions == [subscribe(400, 400 + 20)]

    def test_debounce_duration_throttle_behavior(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, -1), on_next(250, 0), on_next(280, 1),
                                             on_next(310, 2), on_next(350, 3), on_next(400, 4), on_completed(550))
        ys = [scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(on_next(40, 42), on_next(45, 99)), scheduler.create_cold_observable(
            on_next(20, 42), on_next(25, 99)), scheduler.create_cold_observable(on_next(60, 42), on_next(65, 99)), scheduler.create_cold_observable(on_next(20, 42), on_next(25, 99))]

        def create():
            def mapper(x):
                return ys[x]
            return xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 20, 0), on_next(310 + 20, 2), on_next(400 + 20, 4), on_completed(550)]
        assert xs.subscriptions == [subscribe(200, 550)]
        assert ys[0].subscriptions == [subscribe(250, 250 + 20)]
        assert ys[1].subscriptions == [subscribe(280, 310)]
        assert ys[2].subscriptions == [subscribe(310, 310 + 20)]
        assert ys[3].subscriptions == [subscribe(350, 400)]
        assert ys[4].subscriptions == [subscribe(400, 400 + 20)]

    def test_debounce_duration_early_completion(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, -1),
            on_next(250, 0),
            on_next(280, 1),
            on_next(310, 2),
            on_next(350, 3),
            on_next(400, 4),
            on_completed(410))
        ys = [
            scheduler.create_cold_observable(
                on_next(20, 42),
                on_next(25, 99)),
            scheduler.create_cold_observable(
                on_next(40, 42),
                on_next(45, 99)),
            scheduler.create_cold_observable(
                on_next(20, 42),
                on_next(25, 99)),
            scheduler.create_cold_observable(
                on_next(60, 42),
                on_next(65, 99)),
            scheduler.create_cold_observable(
                on_next(20, 42),
                on_next(25, 99))
        ]

        def create():
            def mapper(x):
                return ys[x]
            return xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 20, 0), on_next(310 + 20, 2), on_next(410, 4), on_completed(410)]
        assert xs.subscriptions == [subscribe(200, 410)]
        assert ys[0].subscriptions == [subscribe(250, 250 + 20)]
        assert ys[1].subscriptions == [subscribe(280, 310)]
        assert ys[2].subscriptions == [subscribe(310, 310 + 20)]
        assert ys[3].subscriptions == [subscribe(350, 400)]
        assert ys[4].subscriptions == [subscribe(400, 410)]

    def test_debounce_duration_inner_error(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(
            250, 2), on_next(350, 3), on_next(450, 4), on_completed(550))
        ex = 'ex'

        def create():
            def mapper(x):
                if x < 4:
                    return scheduler.create_cold_observable(on_next(x * 10, "Ignore"), on_next(x * 10 + 5, "Aargh!"))
                else:
                    return scheduler.create_cold_observable(on_error(x * 10, ex))

            return xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 2 * 10, 2), on_next(350 + 3 * 10, 3), on_error(450 + 4 * 10, ex)]
        assert xs.subscriptions == [subscribe(200, 490)]

    def test_debounce_duration_outer_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(
            250, 2), on_next(350, 3), on_next(450, 4), on_error(460, ex))

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_next(x * 10, "Ignore"), on_next(x * 10 + 5, "Aargh!"))
            return  xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 2 * 10, 2), on_next(350 + 3 * 10, 3), on_error(460, ex)]
        assert xs.subscriptions == [subscribe(200, 460)]

    def test_debounce_duration_mapper_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(
            250, 2), on_next(350, 3), on_next(450, 4), on_completed(550))

        def create():
            def mapper(x):
                if x < 4:
                    return scheduler.create_cold_observable(on_next(x * 10, "Ignore"), on_next(x * 10 + 5, "Aargh!"))
                else:
                    _raise(ex)

            return  xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 2 * 10, 2), on_next(350 + 3 * 10, 3), on_error(450, ex)]
        assert xs.subscriptions == [subscribe(200, 450)]

    def test_debounce_duration_inner_done_delay_behavior(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(
            250, 2), on_next(350, 3), on_next(450, 4), on_completed(550))

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_completed(x * 10))
            return  xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(250 + 2 * 10, 2),
            on_next(350 + 3 * 10, 3),
            on_next(450 + 4 * 10, 4),
            on_completed(550)]
        assert xs.subscriptions == [subscribe(200, 550)]

    def test_debounce_duration_inner_done_throttle_behavior(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(250, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(400, 5),
            on_next(410, 6),
            on_completed(550))

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_completed(x * 10))
            return  xs.pipe(_.throttle_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [on_next(250 + 2 * 10, 2), on_next(300 + 4 * 10, 4),
                                    on_next(410 + 6 * 10, 6), on_completed(550)]
        assert xs.subscriptions == [subscribe(200, 550)]



import unittest

from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest


class TestDoWhile(ReactiveTest, unittest.TestCase):

    def test_dowhile_always_false(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            self.on_next(50, 1),
            self.on_next(100, 2),
            self.on_next(150, 3),
            self.on_next(200, 4),
            self.on_completed(250))

        def create():
            return xs.pipe(ops.do_while(lambda _: False))
        results = scheduler.start(create=create)

        assert results.messages == [
            self.on_next(250, 1),
            self.on_next(300, 2),
            self.on_next(350, 3),
            self.on_next(400, 4),
            self.on_completed(450)]
        assert xs.subscriptions == [self.subscribe(200, 450)]

    def test_dowhile_always_true(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            self.on_next(50, 1),
            self.on_next(100, 2),
            self.on_next(150, 3),
            self.on_next(200, 4),
            self.on_completed(250))

        def create():
            return xs.pipe(ops.do_while(lambda _: True))
        results = scheduler.start(create=create)

        assert results.messages == [
            self.on_next(250, 1),
            self.on_next(300, 2),
            self.on_next(350, 3),
            self.on_next(400, 4),
            self.on_next(500, 1),
            self.on_next(550, 2),
            self.on_next(600, 3),
            self.on_next(650, 4),
            self.on_next(750, 1),
            self.on_next(800, 2),
            self.on_next(850, 3),
            self.on_next(900, 4)]
        assert xs.subscriptions == [
            self.subscribe(200, 450),
            self.subscribe(450, 700),
            self.subscribe(700, 950),
            self.subscribe(950, 1000)]

    def test_dowhile_always_true_on_error(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(self.on_error(50, ex))

        def create():
            return xs.pipe(ops.do_while(lambda _: True))
        results = scheduler.start(create=create)

        assert results.messages == [self.on_error(250, ex)]
        assert xs.subscriptions == [self.subscribe(200, 250)]

    def test_dowhile_always_true_infinite(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(self.on_next(50, 1))

        def create():
            return xs.pipe(ops.do_while(lambda _: True))
        results = scheduler.start(create=create)

        assert results.messages == [
            self.on_next(250, 1)]
        assert xs.subscriptions == [self.subscribe(200, 1000)]

    def test_dowhile_sometimes_true(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            self.on_next(50, 1),
            self.on_next(100, 2),
            self.on_next(150, 3),
            self.on_next(200, 4),
            self.on_completed(250))
        n = [0]

        def create():
            def condition(x):
                n[0] += 1
                return n[0]<3
            return xs.pipe(ops.do_while(condition))
        results = scheduler.start(create=create)

        assert results.messages == [
            self.on_next(250, 1),
            self.on_next(300, 2),
            self.on_next(350, 3),
            self.on_next(400, 4),
            self.on_next(500, 1),
            self.on_next(550, 2),
            self.on_next(600, 3),
            self.on_next(650, 4),
            self.on_next(750, 1),
            self.on_next(800, 2),
            self.on_next(850, 3),
            self.on_next(900, 4),
            self.on_completed(950)]
        assert xs.subscriptions == [
            self.subscribe(200, 450),
            self.subscribe(450, 700),
            self.subscribe(700, 950)]

    def test_dowhile_sometimes_throws(self):
        ex = 'ex'
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            self.on_next(50, 1),
            self.on_next(100, 2),
            self.on_next(150, 3),
            self.on_next(200, 4),
            self.on_completed(250))
        n = [0]

        def create():
            def condition(x):
                n[0] += 1
                if n[0]<3:
                    return True
                else:
                    raise Exception(ex)
            return xs.pipe(ops.do_while(condition))
        results = scheduler.start(create=create)

        assert results.messages == [
            self.on_next(250, 1),
            self.on_next(300, 2),
            self.on_next(350, 3),
            self.on_next(400, 4),
            self.on_next(500, 1),
            self.on_next(550, 2),
            self.on_next(600, 3),
            self.on_next(650, 4),
            self.on_next(750, 1),
            self.on_next(800, 2),
            self.on_next(850, 3),
            self.on_next(900, 4),
            self.on_error(950, ex)]
        assert xs.subscriptions == [
            self.subscribe(200, 450),
            self.subscribe(450, 700),
            self.subscribe(700, 950)]



import rx
from rx import scheduler as ccy
from rx import operators as ops

source0 = rx.cold('a-----d---1--------4-|', timespan=0.1)
source1 = rx.cold('--b-c-------2---3-|   ', timespan=0.1)

print("to_marbles() is a blocking operator, we need to wait for completion...")
print('expecting  "a-b-c-d---1-2---3--4-|"')
observable = rx.merge(source0, source1).pipe(ops.to_marbles(timespan=0.1))
diagram = observable.run()
print('got        "{}"'.format(diagram))




import rx
from rx import operators as ops

"""
simple example that merges two cold observables.
"""

source0 = rx.cold('a-----d---1--------4-|', timespan=0.01)
source1 = rx.cold('--b-c-------2---3-|   ', timespan=0.01)

observable = rx.merge(source0, source1).pipe(ops.to_iterable())
elements = observable.run()
print('received {}'.format(list(elements)))



import rx
from rx import operators as ops
from rx.subject import Subject
from rx.scheduler.mainloop import GtkScheduler

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gdk


class Window(Gtk.Window):

    def __init__(self):
        super().__init__()
        self.resize(600, 600)

        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK)
        self.connect('motion-notify-event', self.on_mouse_move)

        self.mousemove = Subject()

    def on_mouse_move(self, widget, event):
        self.mousemove.on_next((event.x, event.y))


def main():
    scheduler = GtkScheduler(GLib)
    scrolled_window = Gtk.ScrolledWindow()

    window = Window()
    window.connect("delete-event", Gtk.main_quit)

    container = Gtk.Fixed()

    scrolled_window.add(container)
    window.add(scrolled_window)
    text = 'TIME FLIES LIKE AN ARROW'

    def on_next(info):
        label, (x, y), i = info
        container.move(label, x + i*12 + 15, y)
        label.show()

    def handle_label(label, i):
        delayer = ops.delay(i*0.100)
        mapper = ops.map(lambda xy: (label, xy, i))

        return window.mousemove.pipe(
            delayer,
            mapper,
        )

    def make_label(char):
        label = Gtk.Label(label=char)
        container.put(label, 0, 0)
        label.hide()
        return label

    mapper = ops.map(make_label)
    labeler = ops.flat_map_indexed(handle_label)

    rx.from_(text).pipe(
        mapper,
        labeler,
    ).subscribe(on_next, on_error=print, scheduler=scheduler)

    window.show_all()

    Gtk.main()


if __name__ == '__main__':
    main()



import unittest
import asyncio
from rx import operators as ops
from rx.subject import Subject

from rx.scheduler.eventloop import AsyncIOScheduler


class TestFlatMapAsync(unittest.TestCase):

    def test_flat_map_async(self):
        actual_next = None
        loop = asyncio.get_event_loop()
        scheduler = AsyncIOScheduler(loop=loop)

        def mapper(i):
            async def _mapper(i):
                return i + 1

            return asyncio.ensure_future(_mapper(i))

        def on_next(i):
            nonlocal actual_next
            actual_next = i

        async def test_flat_map():
            x = Subject()
            x.pipe(ops.flat_map(mapper)).subscribe(on_next, scheduler=scheduler)
            x.on_next(10)
            await asyncio.sleep(0.1)

        loop.run_until_complete(test_flat_map())
        assert actual_next == 11



import unittest

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


def is_even(num):
    return +num % 2 == 0


class TestPartition(unittest.TestCase):
    def test_partition_empty(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_completed(210)
        )

        subscription1 = [None]
        subscription2 = [None]
        observables = []

        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))
        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)
        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()
        assert results1.messages == [
            on_completed(210)]

        assert results2.messages == [
            on_completed(210)]

        assert xs.subscriptions == [
            subscribe(200, 210)]

    def test_partition_single(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_completed(220)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]

        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))
        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)
        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4),
            on_completed(220)]

        assert results2.messages == [
            on_completed(220)]

        assert xs.subscriptions == [
            subscribe(200, 220)]

    def test_partition_each(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_next(220, 3),
            on_completed(230)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]
        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))

        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)

        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()

        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4),
            on_completed(230)]

        assert results2.messages == [
            on_next(220, 3),
            on_completed(230)]

        assert xs.subscriptions == [
            subscribe(200, 230)]

    def test_partition_completed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_next(240, 3),
            on_next(290, 2),
            on_next(350, 1),
            on_completed(360)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]
        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))

        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)
        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4),
            on_next(290, 2),
            on_completed(360)]

        assert results2.messages == [
            on_next(240, 3),
            on_next(350, 1),
            on_completed(360)]

        assert xs.subscriptions == [
            subscribe(200, 360)]

    def test_partition_not_completed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_next(240, 3),
            on_next(290, 2),
            on_next(350, 1)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]

        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))
        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)

        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4),
            on_next(290, 2)]

        assert results2.messages == [
            on_next(240, 3),
            on_next(350, 1)]

        assert xs.subscriptions == [
            subscribe(200, 1000)]

    def test_partition_error(self):
        error = Exception()
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_next(240, 3),
            on_error(290, error),
            on_next(350, 1),
            on_completed(360)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]
        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))
        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)

        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(ReactiveTest.disposed, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4),
            on_error(290, error)]

        assert results2.messages == [
            on_next(240, 3),
            on_error(290, error)]

        assert xs.subscriptions == [
            subscribe(200, 290)]

    def test_partition_disposed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, 5),
            on_next(210, 4),
            on_next(240, 3),
            on_next(290, 2),
            on_next(350, 1),
            on_completed(360)
        )

        observables = []
        subscription1 = [None]
        subscription2 = [None]
        results1 = scheduler.create_observer()
        results2 = scheduler.create_observer()

        def action0(scheduler, state):
            observables.extend(xs.pipe(ops.partition(is_even)))

        scheduler.schedule_absolute(ReactiveTest.created, action0)

        def action1(scheduler, state):
            subscription1[0] = observables[0].subscribe(results1)
            subscription2[0] = observables[1].subscribe(results2)

        scheduler.schedule_absolute(ReactiveTest.subscribed, action1)

        def action2(scheduler, state):
            subscription1[0].dispose()
            subscription2[0].dispose()
        scheduler.schedule_absolute(280, action2)

        scheduler.start()

        assert results1.messages == [
            on_next(210, 4)]

        assert results2.messages == [
            on_next(240, 3)]

        assert xs.subscriptions == [
            subscribe(200, 280)]



import unittest
from datetime import timedelta

import rx
from rx import operators as ops
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TimeInterval(object):
    def __init__(self, value, interval):
        if isinstance(interval, timedelta):
            interval = int(interval.seconds)  # FIXME: Must fix when tests run at fraction of seconds.

        self.value = value
        self.interval = interval

    def __str__(self):
        return "%s@%s" % (self.value, self.interval)

    def equals(self, other):
        return other.interval == self.interval and other.value == self.value


class TestTimeInterval(unittest.TestCase):

    def test_time_interval_regular(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_next(230, 3),
            on_next(260, 4), on_next(300, 5), on_next(350, 6),
            on_completed(400)
        )

        def create():
            def mapper(x):
                return TimeInterval(x.value, x.interval)

            return xs.pipe(
                ops.time_interval(),
                ops.map(mapper),
            )

        results = scheduler.start(create)
        assert results.messages == [
            on_next(210, TimeInterval(2, 10)), on_next(230, TimeInterval(3, 20)),
            on_next(260, TimeInterval(4, 30)), on_next(300, TimeInterval(5, 40)),
            on_next(350, TimeInterval(6, 50)), on_completed(400)]

    def test_time_interval_empty(self):
        scheduler = TestScheduler()

        def create():
            return rx.empty().pipe(ops.time_interval())

        results = scheduler.start(create)
        assert results.messages == [on_completed(200)]

    def test_time_interval_error(self):
        ex = 'ex'
        scheduler = TestScheduler()

        def create():
            return rx.throw(ex).pipe(ops.time_interval())

        results = scheduler.start(create)
        assert results.messages == [on_error(200, ex)]

    def test_time_interval_never(self):
        scheduler = TestScheduler()

        def create():
            return rx.never().pipe(ops.time_interval())

        results = scheduler.start(create)
        assert results.messages == []

    def test_time_interval_default_scheduler(self):
        import datetime
        import time
        xs = rx.of(1, 2).pipe(
            ops.time_interval(),
            ops.pluck_attr('interval'),
        )

        l = []
        d = xs.subscribe(l.append)
        time.sleep(0.1)
        self.assertEqual(len(l), 2)
        [self.assertIsInstance(el, datetime.timedelta) for el in l]



import unittest

import rx
from rx.testing import TestScheduler, ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestFromIterable(unittest.TestCase):
    def test_subscribe_to_iterable_finite(self):
        iterable_finite = [1, 2, 3, 4, 5]
        scheduler = TestScheduler()

        def create():
            return rx.from_(iterable_finite)

        results = scheduler.start(create)

        assert results.messages == [
            on_next(200, 1),
            on_next(200, 2),
            on_next(200, 3),
            on_next(200, 4),
            on_next(200, 5),
            on_completed(200)]

    def test_subscribe_to_iterable_empty(self):
        iterable_finite = []

        scheduler = TestScheduler()

        def create():
            return rx.from_(iterable_finite)
        results = scheduler.start(create)

        assert results.messages == [on_completed(200)]

    def test_double_subscribe_to_iterable(self):
        iterable_finite = [1, 2, 3]
        scheduler = TestScheduler()
        obs = rx.from_(iterable_finite)

        results = scheduler.start(lambda: rx.concat(obs, obs))

        assert results.messages == [
            on_next(200, 1), on_next(200, 2), on_next(200, 3),
            on_next(200, 1), on_next(200, 2), on_next(200, 3),
            on_completed(200)]

    def test_observer_throws(self):
        with self.assertRaises(RxException):
            rx.from_iterable([1, 2, 3]).subscribe(lambda x: _raise('ex'))
